"""
A complex number
"""

import operator

from coerce import bin_op
import complex_field
import element
import real_number
import sage.rings.arith
import sage.misc.misc

class ComplexNumber(element.Element):
    """
    A complex number.
    """
    def __init__(self, real, imag=None, prec=0):
        if imag == None:
            if isinstance(real, list) or isinstance(real,tuple):
                real, imag = real
            else:
                imag = 0
        element.Element.__init__(self, complex_field.CC)
        try:
            self.__re = real_number.RealNumber_mpf(real, prec=prec)
            self.__im = real_number.RealNumber_mpf(imag, prec=prec)
        except TypeError:
            raise TypeError, "unable to coerce (%s,%s) to a ComplexNumber"%(real,imag)
        self.__repr = None
        
    def __repr__(self):
        if self.__repr == None:
            s=""
            if self.__re != 0:
                s=str(self.__re)
            if self.__im != 0:
                y = self.__im
                if s!="":
                    if y < 0:
                        s=s+" - "
                        y=-y
                    else:
                        s=s+" + "
                s=s+"%s*i"%y
            self.__repr = s
        return self.__repr
    
    def __add__(self, right):
        if not isinstance(right, ComplexNumber):
            return bin_op(self, right, operator.add)
        return ComplexNumber(self.__re + right.__re, self.__im + right.__im)
    
    def __radd__(self, left):
        return  ComplexNumber(left) + self
    
    def __sub__(self, right):
        if not isinstance(right, ComplexNumber):
            return bin_op(self, right, operator.sub)
        return ComplexNumber(self.__re - right.__re, self.__im - right.__im)
    
    def __rsub__(self, left):
        return ComplexNumber(left) - self
    
    def __mul__(self, right):
        if not isinstance(right, ComplexNumber):
            return bin_op(self, right, operator.mul)
        return ComplexNumber(self.__re * right.__re - self.__im * right.__im,
                             self.__re * right.__im + self.__im * right.__re)
    
    def __rmul__(self, left):
        return ComplexNumber(left)*self

    def norm(self):
        return self.__re*self.__re + self.__im*self.__im
    
    def __div__(self, right):
        if not isinstance(right, ComplexNumber):
            return bin_op(self, right, operator.div)
        right_nm = right.norm()
        a = right.__re/right_nm
        b = right.__im/right_nm
        re =  a * self.__re + b * self.__im
        im = -b * self.__re + a * self.__im
        return ComplexNumber(re, im)
    
    def __rdiv__(self, left):
        return ComplexNumber(left)/self
    
    def __pow__(self, right):
        return sage.rings.arith.generic_power(self, right)
    
    def __neg__(self):
        return ComplexNumber(-self.__re, -self.__im)
    
    def __pos__(self):
        return self
    
    def __abs__(self):
        x = real_number.RealNumber_mpf(self.__re**2 + self.__im**2, \
                                       prec = self.__re.prec())
        return x.sqrt()
    
    def __invert__(self):
        a = abs(self)
        return ComplexNumber(self.__re/a, -self.__im/a)
    
    def __int__(self):
        return int(self.__value)
    
    def __long__(self):
        return long(self.__value)
    
    def __float__(self):
        return float(self.__value)
    
    def __cmp__(self, other):
        if not isinstance(other, ComplexNumber):
            other = self.parent()(other)
        return sage.misc.misc.generic_cmp((self.__re, self.__im) , (other.__re, other.__im))

    def conjugate(self):
        return ComplexNumber(self.__re, -self.__im)

    def order(self):
        if self.__re == 1 and self.__im == 0:
            return 1
        if self.__re == -1 and self.__im == 0:
            return 2
        raise NotImplementedError, "general order over complex field not implemented."

    def prec(self):
        return self.__re.getprec()

    def real(self):
        return self.__re

    def imag(self):
        return self.__im
