cdef extern from "binary_gcd.h":
    cdef long gcd(long a, long b) except -1
    
######################################################################
# binary GCD alg. Based on following facts.
#   If N and M are even, then gcd(N,M) = 2 * gcd (N/2, M/2)
#   If N even and M odd, then gcd(N,M) = gcd(N/2, M)
#   If N,M odd, then |N-M|<max(N,M) thus guaranteeing that
#             we can use gcd(N,M) = gcd(N-M,M) to reduce.
#   fact: a&1 is 1 iff a is odd. Mult. and Div. by 2 more efficiently
#   implemented as shift-left 1 and shift-right 1 resp.
######################################################################

cdef public int gcd2(int a, int b) except -1:
    cdef int t
    if a < 0:
        a = -a
    if not b:
        return a
    if b<0:
        b = -b
    if not a:
        return b
    t = 0
    while not ((a|b) & 1):
        a = a >> 1
        b = b >> 1
        t = t + 1
    while not (a&1):
        a = a >> 1                     
    while not (b&1):
        b = b >> 1
    while a != b:
        if a > b:
            a = a - b
            a = a >> 1
            while not (a&1):
                a = a >> 1
        else:
            b = b - a
            b = b >> 1
            while not (b&1):
                b = b >> 1
    return a<<t

import time
def test_gcd(int n, long a, long b):
    cdef int i
    t = time.clock()
    for i from 0 <= i < n:
        gcd(a,b)
    print time.clock() - t
    t = time.clock()
    for i from 0 <= i < n:
        gcd2(a,b)
    print time.clock() - t        
    
