"""
Abstract base class for ring elements.
"""

##################################################################
# Generic ring element, so all this functionality must be defined
# by any ring element.  Derived class must call __init__
##################################################################

import element_py

# This classes uses element.pxd.  To add data members, you
# must change that file. 

cdef class Element:
    """
    Generic ring element base class, so all this functionality must
    be defined by any ring element.
    """

    def __init__(self, parent, required_instance=None):
        self.__parent = parent
        if required_instance != None:
            assert isinstance(parent, required_instance), \
               "Parent must be of type %s"%required_instance

    def __reduce__(self):
        return element_py.init, (self.__parent,)

    def __repr__(self):
        return "Generic element of a ring"

    def __hash__(self):
        return hash(str(self))

    def order(self):
        raise NotImplementedError

    def is_zero(self):
        return self == self.parent()(0)
    
    def parent(self, x=None):
        if x==None:
            return self.__parent
        else:
            return self.__parent(x)

    def __xor__(self, right):
        raise RuntimeError, "Use ** for exponentiation, not '^', which means xor\n"+\
              "in Python, and has the wrong precedence."

