"""
Matrix base class (extension type).
"""

import matrix_space

cdef class Matrix:
    cdef object __parent
    cdef int __nrows
    cdef int __ncols
    
    def __init__(self, parent):
        if not isinstance(parent, matrix_space.MatrixSpace):
            raise TypeError, "parent must be a matrix_space.MatrixSpace"
        self.__parent = parent
        self.__nrows = parent.nrows()
        self.__ncols = parent.ncols()
        
    def __repr__(self):
        return "Element of %s"%self.parent()
        
    def __str__(self):
        return "%s"%self
    
    ###################################################
    ## Access
    ###################################################
    def __getitem__(self, ij):
        raise NotImplementedError
    
    def __setitem__(self, ij, x):
        raise NotImplementedError
        
    ###################################################
    ## Properties
    ###################################################
    def base_ring(self):
        return self.__parent.base_ring()
        
    def ncols(self):
        return self.__ncols
    
    def nrows(self):
        return self.__nrows
    
    def parent(self):
        return self.__parent

    ###################################################
    ## Functions
    ###################################################
    def augment(self, A):
        raise NotImplementedError
        
    def block_sum(self, matrix):
        raise NotImplementedError
        
    def change_ring(self, ring):
        raise NotImplementedError
        
    def characteristic_polynomial(self):
        raise NotImplementedError
        
    def column_space(self):
        raise NotImplementedError
        
    def commutator(self, other):
        raise NotImplementedError
      
    def copy(self):
        raise NotImplementedError
        
    def denominator(self):
        raise NotImplementedError
        
    def determinant(self):
        raise NotImplementedError
        
    def height(self):
        raise NotImplementedError
        
    def hermite_normal_form(self):
        raise NotImplementedError
        
    def hessenberg_normal_form(self):
        raise NotImplementedError
        
    def image(self):
        raise NotImplementedError
    
    def is_invertible(self):
        raise NotImplementedError
        
    def kernel(self):
        raise NotImplementedError
        
    def list(self):
        raise NotImplementedError
        
    def nullity(self):
        raise NotImplementedError
        
    def pivots(self):
        raise NotImplementedError
        
    def rank(self):
        raise NotImplementedError
        
    def rational_decomposition(self):
        raise NotImplementedError
        
    def rational_reconstruction(self):
        raise NotImplementedError
        
    def reduced_echelon_form(self):
        raise NotImplementedError
        
    def restrict(self, V):
        raise NotImplementedError
        
    def rows(self):
        raise NotImplementedError
        
    def row_space(self):
        raise NotImplementedError
        
    def smith_normal_form(self):
        raise NotImplementedError
        
    def stack(self, matrix):
        raise NotImplementedError
        
    def swap_columns(self, c1, c2):
        raise NotImplementedError
        
    def trace(self):
        if self.nrows() != self.ncols():
            raise ArithmeticError, "matrix must be square."
        cdef int i, nr
        s = self.base_ring()(0)
        nr = self.nrows()
        for i from 0 <= i < nr:
            s = s + self[i,i]
        return s
        
    def transpose(self):
        raise NotImplementedError
        
    ###################################################
    ## Arithmetic
    ###################################################
    def __abs__(self):
        return self.determinant()
    
    def __add__(self, other):
        raise NotImplementedError
        
    def __cmp__(self, right):
        raise NotImplementedError
        
    def __invert__(self):
        raise NotImplementedError
        
    def __mod__(self, p):
        raise NotImplementedError
    
    def __mul__(Matrix self, Matrix right):
        raise NotImplementedError
        
    def __neg__(self):
        raise NotImplementedError
        
    def __pos__(self):
        return self
        
    def __pow__(self, n, i):
        raise NotImplementedError
        
    def __sub__(self, right):
        raise NotImplementedError
    
