"""
pari_pyx.pyx
"""

import manin.misc.misc

include 'cdefs.pxi'

cdef extern from 'pari/pari.h':
    ctypedef long* GEN
    GEN     flisseq(char *t)
    char*   GENtostr(GEN x)
    void    cgiv(GEN x)
    ctypedef int size_t
    void    pari_init(size_t parisize, unsigned long maxprime)
    void    pari_sig_init(...)
    
initialized = False

# cdef int gp_exception_handler(long numerr):
#     return 0
#     cdef char *s
#     cdef int recovering
#     s = <char*>global_err_data;
#     if s != <char *>0:
#         return 0
#     if s[0]:
#         recovering = 1
#         fprintferr("\n")
#         outerr(lisseq(s))
#         recovering = 0
#         return 0
    #if numerr == errpile:
    #    var_make_safe()
    #    avma = top
    #return break_loop(numerr)


def init(long size=100000000, unsigned long maxprime=50000):
    """
    Initialize the PARI system.
    
    INPUT:
        size -- long, the number of bytes given to PARI to work with
                (default: 100000000)
        maxprime -- unsigned long, upper limit on a precomputed prime
                    number table  (default: 50000)
    
    If you do not want prime numbers, put maxprime=2, but be careful
    because many PARI functions require this table.  If you get the
    error message "not enough precomputed primes", increase this
    parameter.
    EXAMPLES:
        >>> pari_init()   
    """
    global initialized
    if not initialized:
        pari_init(size, maxprime);
        initialized = True
    else:
        raise RuntimeError, "pari_init has already been called."

def execute(s):
    manin.misc.misc.verbose("Pari command: "+s)
    if not initialized:
        init()
    # Parse s in PARI.
    s = s + "\n" 
    flisseq(s)
    
def eval(s):
    manin.misc.misc.verbose("Pari command: "+s)
    if not initialized:
        init()
    cdef GEN x
    cdef char *c
    # Parse s in pari and evaluate result.
    s = s + "\n"
    x = flisseq(s)
    # Get result as a C-string
    c = GENtostr(x)
    # De-callocate memory used by x:
    cgiv(x)
    # Convert c to a Python string.
    s = str(c)
    # Free memory used by c
    free(c)
    return s
    
